/**
 * 
 */
package com.softslate.commerce.administrator.importexport;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.customer.core.BaseAction;

import com.softslate.commerce.businessobjects.importexport.SalesReportProcessor;

/**
 * Struts Action class responsible for producing the Sales By Product report.
 * 
 * @author David Tobey
 */
public class SalesByProductAction extends BaseAction {

	static Log log = LogFactory.getLog(SalesByProductAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		SalesByProductForm baseForm = (SalesByProductForm) form;
		Map parameters = PropertyUtils.describe(baseForm);

		synchronized (request.getSession()) {
			try {

				BusinessObjectFactory bof = (BusinessObjectFactory) request
						.getAttribute("businessObjectFactory");
				SalesReportProcessor srp =  bof
						.createObject(SalesReportProcessor.class);

				Collection results = srp
						.processSalesByProductReport(parameters);
				request.setAttribute("salesByProduct", results);

				// Output to screen
				if (baseForm.getOutputFormat().equals("screen")) {

					// Create breadcrumbing collection
					Collection bcs = Arrays.asList(new String[] { "Home",
							"Sales by Product Report Form",
							"Sales by Product Report" });
					request.setAttribute("breadcrumbs", bcs);

					// On-screen output
					return mapping.findForward("success");
				}

				// File download output
				String outputFormat = baseForm.getOutputFormat();

				// Figure out the delimiter and the file extension
				String characterEncoding = (String) baseForm.getSettings()
						.getValue("characterEncoding");
				if (characterEncoding == null
						|| characterEncoding.length() == 0) {
					characterEncoding = "utf-8";
				}
				String fileExt = "txt"; // default
				char delimiter = '\t'; // default
				if (outputFormat.equalsIgnoreCase(ExportForm.OUTPUT_FORMAT_CSV)) {
					fileExt = "csv";
					delimiter = ',';
				} else if (outputFormat
						.equalsIgnoreCase(ExportForm.OUTPUT_FORMAT_PIPE)) {
					fileExt = "txt";
					delimiter = '|';
				}

				// Put the delimiter in the request for the JSP to use
				request.setAttribute("delimiter", new Character(delimiter));

				// Find the filename to tell the browser to use.
				String filename = "salesByProduct" + "_"
						+ baseForm.getStartDate() + "_" + baseForm.getEndDate()
						+ "." + fileExt;
				if (log.isDebugEnabled())
					log.debug("filename " + filename);

				// Set the response headers
				response.setContentType("application/vnd.ms-excel;charset="
						+ characterEncoding);
				response.setHeader("Content-Disposition",
						"attachment; filename=" + filename);
				return mapping.findForward("successCSV");

			} catch (Exception ex) {
				log.error(ex.getMessage(), ex);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.detail", ex.getMessage()));
				saveErrors(request, (ActionMessages) baseForm.getErrors());

				return mapping.findForward("failure");
			}
		}
	}

}
