package com.softslate.commerce.administrator.order;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.email.Template;
import com.softslate.commerce.businessobjects.email.TemplateProcessor;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * orders. Used for requests to "/administrator/Order.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 * @author David Tobey
 * 
 */
public class OrderAction extends BaseAction {

	static Log log = LogFactory.getLog(OrderAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		OrderForm baseForm = (OrderForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		OrderProcessor processor = bof.createObject(OrderProcessor.class);
		Map ordersAndCount = processor.loadOrdersAndCount(PropertyUtils.describe(baseForm));
		request.setAttribute("orders", ordersAndCount.get("items"));
		request.setAttribute("itemCollection", ordersAndCount.get("items"));
		request.setAttribute("itemCount", ordersAndCount.get("itemCount"));
		request.setAttribute("assignedItems", ordersAndCount.get("assignedItems"));

		// Load all Templates for the email template drop down.
		if (baseForm.getEditMode() == null || baseForm.getEditMode().equals("false")) {
			try {
				TemplateProcessor tp = bof.createObject(TemplateProcessor.class);
				Collection<Template> templates = tp.loadAllTemplates();
				request.setAttribute("templates", templates);
			} catch (Exception e) {
				log.info("Cannot load email templates: ", e);
			}
		}

		// Load the parent if an id is present
		Customer customer = null;
		if (baseForm.getCustomerID() != null && baseForm.getCustomerID().length() > 0) {
			CustomerProcessor customerProcessor = bof.createObject(CustomerProcessor.class);
			customer = customerProcessor.loadCustomerFromID(PropertyUtils.describe(baseForm));
			request.setAttribute("customer", customer);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Orders" });
		request.setAttribute("breadcrumbs", bcs);

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=orders.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
