package com.softslate.commerce.administrator.product;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.LuceneIndexer;
import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/CategoryEdit.do". Processes a request to edit a record within
 * the system.
 * 
 * @author David Tobey
 */
public class CategoryEditAction extends BaseAction {

	static Log log = LogFactory.getLog(CategoryEditAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		CategoryAddEditForm baseForm = (CategoryAddEditForm) form;

		// Try uploading any files, these will throw exceptions if they do not
		// work therefore the product will not be added in that case.
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		FileStorageProcessor fsp = (FileStorageProcessor) baseForm.getBusinessObjectFactory().createObject(
				"fileStorageProcessorImplementer");

		// Load the imageSystemPath and imagePathURL settings; derive the
		// imageSystemPath from the URL path if blank.
		String imageSystemPath = baseForm.getSettings().getValue("imageSystemPath").toString();
		String imagePathURL = baseForm.getSettings().getValue("imagePathURL").toString();
		if (!imagePathURL.endsWith("/")) {
			imagePathURL = imagePathURL + "/";
		}
		if (imageSystemPath == null || imageSystemPath.length() == 0) {
			imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
					+ System.getProperty("file.separator");
		}
		if (!imageSystemPath.endsWith(System.getProperty("file.separator"))) {
			imageSystemPath = imageSystemPath + System.getProperty("file.separator");
		}
		imagePathURL = baseForm.getRequest().getContextPath() + imagePathURL;
		String categoryOrContent = "Category";
		if (baseForm.getIsContentPage() != null && baseForm.getIsContentPage().equals("true")) {
			categoryOrContent = "Content";
		}
		if (baseForm.getSmallImageUpload() != null && baseForm.getSmallImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getSmallImageUpload().getFileName());
			fsp.storeFile(baseForm.getSmallImageUpload().getInputStream(), imageSystemPath + filename, "small"
					+ categoryOrContent);
			baseForm.setSmallImage(fsp.urlForFile(imagePathURL, filename, "small" + categoryOrContent));
		}
		if (baseForm.getMediumImageUpload() != null && baseForm.getMediumImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getMediumImageUpload().getFileName());
			fsp.storeFile(baseForm.getMediumImageUpload().getInputStream(), imageSystemPath + filename, "medium"
					+ categoryOrContent);
			baseForm.setMediumImage(fsp.urlForFile(imagePathURL, filename, "medium" + categoryOrContent));
		}
		if (baseForm.getLargeImageUpload() != null && baseForm.getLargeImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(baseForm.getLargeImageUpload().getFileName());
			fsp.storeFile(baseForm.getLargeImageUpload().getInputStream(), imageSystemPath + filename, "large"
					+ categoryOrContent);
			baseForm.setLargeImage(fsp.urlForFile(imagePathURL, filename, "large" + categoryOrContent));
		}
		
		// Handle generating the code from the name if it is empty
		if ((baseForm.getCode() == null || baseForm.getCode().equals(""))
				&& (baseForm.getName() != null && !baseForm.getName().equals(""))) {
			baseForm.setCode(actionUtils.generateCodeFromName(baseForm.getName()));
		}

		// Edit the record
		CategoryProcessor processor = (CategoryProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"categoryProcessorImplementer");
		Map results = processor.editCategory(baseForm.mapOfIncomingParams());

		// Load all categories for the parent category drop down.
		Collection categories = processor.loadAllCategories();
		baseForm.getRequest().setAttribute("categories", categories);

		// Successful update
		if (results == null || results.get("resultCode") == null
				|| results.get("resultCode").equals("0")) {

			// entity.jsp wants the updated record in the request scope
			Category category = (Category) results.get("updatedRecord");
			request.setAttribute("category", category);

			// Reload category tree and builtin categories in the application
			// scope.
			ProductInitializer pi = new ProductInitializer();
			pi.setServlet(servlet);
			pi.initializeCategoryTree(baseForm.getBusinessObjectFactory());
			pi.initializeBuiltInCategories(baseForm.getBusinessObjectFactory());

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.recordSaved"));

			// Optionally update the Lucene index
			updateAdvancedSearchIndex(baseForm, category.getCategoryID());
		} else if (results != null && results.get("existingRecord") != null) {
			Category existingRecord = (Category) results.get("existingRecord");
			String existingCode = existingRecord.getCode();
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode"
							+ results.get("resultCode"), results
							.get("duplicateField"), existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord",
							"CategoryEditForm.do?categoryID="
									+ existingRecord.getCategoryID(), results
									.get("duplicateField"), existingCode));
		}

		// Save errors and messages
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");
	}

	protected void updateAdvancedSearchIndex(CategoryAddEditForm baseForm, int categoryID) {
		if (baseForm.getSettings().getValue("advancedSearchOn").equals("1") && baseForm.getUpdateAdvancedSearchIndex()) {
			Map indexerResults = null;
			try {
				LuceneIndexer indexer = baseForm.getBusinessObjectFactory().createObject(LuceneIndexer.class);
				String index = (String) baseForm.getSettings().getValue("luceneIndexDir");
				String indexDir = getServlet().getServletContext().getRealPath(index);
				indexerResults = indexer.addEditCategory(categoryID, indexDir);
				if (indexerResults.get("message") != null && ((String) indexerResults.get("message")).length() > 0) {
					baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", indexerResults.get("message")));
				}
			} catch (Exception e) {
				log.error("Exception reindexing category: ", e);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.detail", e.toString()));
			}
		}
	}
}
