package com.softslate.commerce.businessobjects.administrator;

import java.io.Serializable;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing an administrator within the system.
 * <code>AdministratorBean</code> is the default implementation of the
 * <code>Administrator</code> interface for the application.
 * <p>
 * The database stores administrators in the <code>sscAdministrator</code>
 * table, and this class can be used to represent a single row of that table.
 * The Hibernate configuration file <code>AdministratorBean.hbm.xml</code>
 * maps that table's columns to the properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudAdministrator")
public class AdministratorBean extends BaseBusinessObject implements
		Administrator, Serializable {

	private static final long serialVersionUID = 4049359703806980917L;

	static Log log = LogFactory.getLog(AdministratorBean.class);

	private int administratorID = 0;

	public int getAdministratorID() {
		return (this.administratorID);
	}

	public void setAdministratorID(int administratorID) {
		this.administratorID = administratorID;
	}

	private String userName = null;

	public String getUserName() {
		return (this.userName);
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	private String password = null;

	public String getPassword() {
		return (this.password);
	}

	public void setPassword(String password) {
		this.password = password;
	}

	private String decryptedPassword = null;

	public String getDecryptedPassword() {
		return (this.decryptedPassword);
	}

	public void setDecryptedPassword(String decryptedPassword) {
		this.decryptedPassword = decryptedPassword;
	}

	private String lastLogin = null;

	public String getLastLogin() {
		return (this.lastLogin);
	}

	public void setLastLogin(String lastLogin) {
		this.lastLogin = lastLogin;
	}

	private boolean isApiUser = false;

	public boolean getIsApiUser() {
		return isApiUser;
	}

	public void setIsApiUser(boolean isApiUser) {
		this.isApiUser = isApiUser;
	}
	
	private String passwordExpires = null;

	public String getPasswordExpires() {
		return passwordExpires;
	}

	public void setPasswordExpires(String passwordExpires) {
		this.passwordExpires = passwordExpires;
	}
	
	private Integer numberOfFailedLogins = null;

	public Integer getNumberOfFailedLogins() {
		return numberOfFailedLogins;
	}

	public void setNumberOfFailedLogins(Integer numberOfFailedLogins) {
		this.numberOfFailedLogins = numberOfFailedLogins;
	}
	
	private String lockedOutUntil = null;

	public String getLockedOutUntil() {
		return lockedOutUntil;
	}

	public void setLockedOutUntil(String lockedOutUntil) {
		this.lockedOutUntil = lockedOutUntil;
	}
	
	private String previousPasswords = null;

	public String getPreviousPasswords() {
		return previousPasswords;
	}

	public void setPreviousPasswords(String previousPasswords) {
		this.previousPasswords = previousPasswords;
	}

	private String allowedIPAddresses = null;

	public String getAllowedIPAddresses() {
		return allowedIPAddresses;
	}

	public void setAllowedIPAddresses(String allowedIPAddresses) {
		this.allowedIPAddresses = allowedIPAddresses;
	}

	private boolean isActive;

	public boolean getIsActive() {
		return (this.isActive);
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	private Collection roleCodes = null;

	public Collection getRoleCodes() {
		return (this.roleCodes);
	}

	public void setRoleCodes(Collection roleCodes) {
		this.roleCodes = roleCodes;
	}

	private int importID = 0;

	public int getImportID() {
		return (this.importID);
	}

	public void setImportID(int importID) {
		this.importID = importID;
	}

	private Collection exportedAttributeIDs = null;

	public Collection getExportedAttributeIDs() {
		return exportedAttributeIDs;
	}

	public void setExportedAttributeIDs(Collection exportedAttributeIDs) {
		this.exportedAttributeIDs = exportedAttributeIDs;
	}

	private Collection administratorRoles = null;

	public Collection getAdministratorRoles() {
		return administratorRoles;
	}

	public void setAdministratorRoles(Collection administratorRoles) {
		this.administratorRoles = administratorRoles;
	}

}
