package com.softslate.commerce.businessobjects.order;

import java.util.Collection;
import java.util.Locale;
import java.util.Map;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.customer.Customer;

/**
 * Interface representing an order within the system. An order may be complete
 * and part of a customer's order history, or it may represent the user's cart:
 * in other words, the current order the customer is building in the store.
 * <p>
 * When the application needs to create an instance that implements
 * <code>Order</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "orderImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "orderImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderBean}.
 * 
 * @author David Tobey
 * 
 * 
 */
@Audited
@AuditTable("AudOrder")
public interface Order extends BusinessObject {
	abstract int getOrderID();

	abstract void setOrderID(int orderID);

	abstract Integer getCustomerID();

	abstract void setCustomerID(Integer customerID);

	abstract String getUserName();

	abstract void setUserName(String userName);

	abstract Integer getOrderNumber();

	abstract void setOrderNumber(Integer orderNumber);

	abstract String getTrackingNumber();

	abstract void setTrackingNumber(String trackingNumber);

	abstract Double getTotal();

	abstract void setTotal(Double total);

	abstract Double getShipping();

	abstract void setShipping(Double shipping);

	abstract Double getTax();

	abstract void setTax(Double tax);

	abstract Double getTaxableSubtotal();

	abstract void setTaxableSubtotal(Double taxableSubtotal);

	abstract Double getSubtotal();

	abstract void setSubtotal(Double subtotal);

	abstract Double getWeight();

	abstract void setWeight(Double weight);

	abstract Integer getQuantity();

	abstract void setQuantity(Integer quantity);

	abstract String getFormattedTotal();

	abstract void setFormattedTotal(String formattedTotal);

	abstract String getFormattedShipping();

	abstract void setFormattedShipping(String formattedShipping);

	abstract String getFormattedTax();

	abstract void setFormattedTax(String formattedTax);

	abstract String getFormattedTaxableSubtotal();

	abstract void setFormattedTaxableSubtotal(String formattedTaxableSubtotal);

	abstract String getFormattedSubtotal();

	abstract void setFormattedSubtotal(String formattedSubtotal);

	abstract String getCreated();

	abstract void setCreated(String created);

	abstract String getLastModified();

	abstract void setLastModified(String lastModified);

	abstract String getCompleted();

	abstract void setCompleted(String completed);

	abstract String getFormattedCreated();

	abstract void setFormattedCreated(String formattedCreated);

	abstract String getFormattedLastModified();

	abstract void setFormattedLastModified(String formattedLastModified);

	abstract String getFormattedCompleted();

	abstract void setFormattedCompleted(String formattedCompleted);

	abstract String getStatus();

	abstract void setStatus(String status);

	abstract String getStatusDetails();

	abstract void setStatusDetails(String statusDetails);

	abstract String getShippingMethod();

	abstract void setShippingMethod(String shippingMethod);

	abstract String getFirstName();

	abstract void setFirstName(String firstName);

	abstract String getLastName();

	abstract void setLastName(String lastName);

	abstract String getOrganization();

	abstract void setOrganization(String organization);

	abstract String getAddress1();

	abstract void setAddress1(String address1);

	abstract String getAddress2();

	abstract void setAddress2(String address2);

	abstract String getCity();

	abstract void setCity(String city);

	abstract String getState();

	abstract void setState(String state);

	abstract String getCountry();

	abstract void setCountry(String country);

	abstract String getPostalCode();

	abstract void setPostalCode(String postalCode);

	abstract String getPhone1();

	abstract void setPhone1(String phone1);

	abstract String getPhone2();

	abstract void setPhone2(String phone2);

	abstract String getEmail1();

	abstract void setEmail1(String email1);

	abstract String getEmail2();

	abstract void setEmail2(String email2);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getExtra4();

	abstract void setExtra4(String extra4);

	abstract String getExtra5();

	abstract void setExtra5(String extra5);

	abstract String getNotes();

	abstract void setNotes(String notes);

	abstract String getAutoSavedCartToken();

	abstract void setAutoSavedCartToken(String autoSavedCartToken);

	abstract Collection getOrderDeliveryCollection();

	abstract void setOrderDeliveryCollection(Collection orderDeliveryCollection);

	@NotAudited
	abstract Collection getOrderItemCollection();

	abstract void setOrderItemCollection(Collection orderItemCollection);

	abstract Collection getPayments();

	abstract void setPayments(Collection payments);

	abstract Collection getOrderDiscounts();

	abstract void setOrderDiscounts(Collection orderDiscounts);

	abstract Customer getCustomer();

	abstract void setCustomer(Customer customer);

	abstract String getBillingFirstName();

	abstract void setBillingFirstName(String billingFirstName);

	abstract String getBillingLastName();

	abstract void setBillingLastName(String billingLastName);

	abstract String getBillingOrganization();

	abstract void setBillingOrganization(String billingOrganization);

	abstract String getBillingAddress1();

	abstract void setBillingAddress1(String billingAddress1);

	abstract String getBillingAddress2();

	abstract void setBillingAddress2(String billingAddress2);

	abstract String getBillingCity();

	abstract void setBillingCity(String billingCity);

	abstract String getBillingState();

	abstract void setBillingState(String billingState);

	abstract String getBillingCountry();

	abstract void setBillingCountry(String billingCountry);

	abstract String getBillingPostalCode();

	abstract void setBillingPostalCode(String billingPostalCode);

	abstract String getBillingPhone1();

	abstract void setBillingPhone1(String billingPhone1);

	abstract String getBillingPhone2();

	abstract void setBillingPhone2(String billingPhone2);

	abstract String getBillingEmail1();

	abstract void setBillingEmail1(String billingEmail1);

	abstract String getBillingEmail2();

	abstract void setBillingEmail2(String billingEmail2);

	abstract String getBillingExtra1();

	abstract void setBillingExtra1(String billingExtra1);

	abstract String getBillingExtra2();

	abstract void setBillingExtra2(String billingExtra2);

	abstract String getBillingExtra3();

	abstract void setBillingExtra3(String billingExtra3);

	abstract String getBillingExtra4();

	abstract void setBillingExtra4(String billingExtra4);

	abstract String getBillingExtra5();

	abstract void setBillingExtra5(String billingExtra5);

	abstract String getBillingNotes();

	abstract void setBillingNotes(String billingNotes);

	abstract OrderDelivery firstDelivery();

	abstract void addFormatting(Locale locale, String currencyCode);

	abstract Collection getOrderShippingRules();

	abstract void setOrderShippingRules(Collection orderShippingRules);

	abstract Collection getEstimatedShippingOptions();

	abstract void setEstimatedShippingOptions(
			Collection estimatedShippingOptions);

	abstract String getPaymentProfileID();

	abstract void setPaymentProfileID(String paymentProfileID);

	abstract boolean getSkipAddressValidation();

	abstract void setSkipAddressValidation(boolean skipAddressValidation);

	abstract Map getOrderSettings();

	abstract void setOrderSettings(Map orderSettings);
}
