package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Map;

import org.apache.lucene.facet.search.FacetsCollector;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for classes that search on the Lucene index for facets.
 * 
 * When the application needs to create an instance that implements
 * <code>LuceneFacetSearcher</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "luceneFacetSearcherImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "luceneFacetHelperImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.BasicLuceneFacetHelper}.
 * 
 * @author David Tobey
 * 
 */
public interface LuceneFacetHelper extends BusinessProcessor {

	public static char FACET_SEPARATOR = '|';

	public abstract BooleanQuery createFacetsBooleanQuery(String[] facets);

	public abstract Collection performFacetSearch(Query mainQuery, SearchList searchList, IndexSearcher searcher)
			throws Exception;

	public abstract FacetsCollector createFacetsCollector(SearchList searchList) throws Exception;

	public abstract SearchList loadFacetCategoriesAndManufacturers(SearchList searchList) throws Exception;

	public abstract Map loadFacetManufacturersFromDatabase(Collection facetResultCollection) throws Exception;

	public abstract void logFacetResults(Collection facetResultCollection);

	public abstract Collection simplifyFacetResults(Collection facetResults);

	public abstract Map simplifyFacetResultsAsMap(Collection facetResults);
	

}
