package com.softslate.commerce.businessobjects.product;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for classes that search on the Lucene index for advanced search
 * functionality.
 * 
 * When the application needs to create an instance that implements
 * <code>LuceneSearcher</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "luceneSearcherImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "luceneSearcherImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.BasicLuceneSearcher}.
 * 
 * @author Jason McWilliams
 * 
 */
public interface LuceneSearcher extends BusinessProcessor {

	abstract SearchList getSearchList(SearchList searchList) throws Exception;

	abstract SearchList getFacetCounts(SearchList searchList) throws Exception;
}
