package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;
import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a sku within the system. A sku corresponds to a
 * {@link com.softslate.commerce.businessobjects.product.Product}, a
 * <code>Product</code> and any combination of
 * {@link com.softslate.commerce.businessobjects.product.Attribute}s and
 * {@link com.softslate.commerce.businessobjects.product.Option}s, or an
 * <code>Attribute</code> and its <code>Option</code>s, independent of any
 * products. It is used for tracking inventory and can it also be associated
 * with discounts. (SKU stands for "stock keeping unit".) <code>SKUBean</code>
 * is the default implementation of the <code>SKU</code> interface for the
 * application.
 * <p>
 * The database stores skus in the <code>sscSKU</code> table, and this
 * interface can be used to represent a single row of that table. The Hibernate
 * configuration file <code>SKUBean.hbm.xml</code> maps that table's columns
 * to the properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudSKU")
public class SKUBean extends BaseBusinessObject implements SKU, Serializable {

	private static final long serialVersionUID = 4650574269639320317L;

	private int skuID = 0;

	public int getSKUID() {
		return skuID;
	}

	public void setSKUID(int skuID) {
		this.skuID = skuID;
	}

	private Integer productID = null;

	private Integer attributeID1 = null;

	private Integer attributeID2 = null;

	private Integer attributeID3 = null;

	private Integer attributeID4 = null;

	private Integer attributeID5 = null;

	private Integer optionID1 = null;

	private Integer optionID2 = null;

	private Integer optionID3 = null;

	private Integer optionID4 = null;

	private Integer optionID5 = null;

	private String skuName = null;

	private String skuNumber = null;

	private String skuMessage = null;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private boolean trackInventory = true;

	private int inventoryLevel = 0;

	private String notes = null;

	private Product product = null;

	private Collection skuDiscounts = null;
	
	private Collection skuShippingRules = null;

	public Integer getAttributeID1() {
		return attributeID1;
	}

	public void setAttributeID1(Integer attributeID1) {
		this.attributeID1 = attributeID1;
	}

	public Integer getAttributeID2() {
		return attributeID2;
	}

	public void setAttributeID2(Integer attributeID2) {
		this.attributeID2 = attributeID2;
	}

	public Integer getAttributeID3() {
		return attributeID3;
	}

	public void setAttributeID3(Integer attributeID3) {
		this.attributeID3 = attributeID3;
	}

	public Integer getAttributeID4() {
		return attributeID4;
	}

	public void setAttributeID4(Integer attributeID4) {
		this.attributeID4 = attributeID4;
	}

	public Integer getAttributeID5() {
		return attributeID5;
	}

	public void setAttributeID5(Integer attributeID5) {
		this.attributeID5 = attributeID5;
	}

	public int getInventoryLevel() {
		return inventoryLevel;
	}

	public void setInventoryLevel(int inventoryLevel) {
		this.inventoryLevel = inventoryLevel;
	}

	public String getNotes() {
		return notes;
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}

	public Integer getOptionID1() {
		return optionID1;
	}

	public void setOptionID1(Integer optionID1) {
		this.optionID1 = optionID1;
	}

	public Integer getOptionID2() {
		return optionID2;
	}

	public void setOptionID2(Integer optionID2) {
		this.optionID2 = optionID2;
	}

	public Integer getOptionID3() {
		return optionID3;
	}

	public void setOptionID3(Integer optionID3) {
		this.optionID3 = optionID3;
	}

	public Integer getOptionID4() {
		return optionID4;
	}

	public void setOptionID4(Integer optionID4) {
		this.optionID4 = optionID4;
	}

	public Integer getOptionID5() {
		return optionID5;
	}

	public void setOptionID5(Integer optionID5) {
		this.optionID5 = optionID5;
	}

	public Integer getProductID() {
		return productID;
	}

	public void setProductID(Integer productID) {
		this.productID = productID;
	}

	public int getSkuID() {
		return skuID;
	}

	public void setSkuID(int skuID) {
		this.skuID = skuID;
	}

	public String getSkuNumber() {
		return skuNumber;
	}

	public void setSkuNumber(String skuNumber) {
		this.skuNumber = skuNumber;
	}

	public boolean getTrackInventory() {
		return trackInventory;
	}

	public void setTrackInventory(boolean trackInventory) {
		this.trackInventory = trackInventory;
	}

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public String getExtra1() {
		return extra1;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getSkuName() {
		return skuName;
	}

	public void setSkuName(String skuName) {
		this.skuName = skuName;
	}

	public String getSkuMessage() {
		return skuMessage;
	}

	public void setSkuMessage(String skuMessage) {
		this.skuMessage = skuMessage;
	}

	public Collection getSkuDiscounts() {
		return skuDiscounts;
	}

	public void setSkuDiscounts(Collection skuDiscounts) {
		this.skuDiscounts = skuDiscounts;
	}
	
	public Collection getSkuShippingRules() {
		return skuShippingRules;
	}

	public void setSkuShippingRules(Collection skuShippingRules) {
		this.skuShippingRules = skuShippingRules;
	}
}
