package com.softslate.commerce.businessobjects.shipping;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface for business logic methods having to do with shipping methods.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting shipping methods.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ShippingMethodProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "shippingMethodProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "shippingMethodProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.BasicShippingMethodProcessor}.
 * 
 * @author David Tobey
 */
public interface ShippingMethodProcessor extends BusinessObject {

	abstract Map loadShippingMethodsAndCount(Map parameters) throws Exception;

	abstract Map addShippingMethod(Map parameters) throws Exception;

	abstract ShippingMethod loadShippingMethodFromID(Map parameters)
			throws Exception;

	abstract ShippingMethod loadShippingMethodFromCode(Map parameters)
			throws Exception;

	abstract Map editShippingMethod(Map parameters) throws Exception;

	abstract Map editDeleteShippingMethods(Map parameters) throws Exception;

	abstract void deleteShippingMethod(Map parameters) throws Exception;
}
