package com.softslate.commerce.businessobjects.shipping;

import java.util.Comparator;
import java.util.Map;

/**
 * Comparator class used to sort shipping options based on their price, from
 * lowest to highest.
 * 
 * @author David Tobey
 * 
 */
public class ShippingOptionComparator implements Comparator {

	public int compare(Object o1, Object o2) {
		try {
			Map option1 = (Map) o1;
			Map option2 = (Map) o2;

			// First sort by the order field - which may or may not be defined
			Integer order1 = (Integer) option1.get("methodOrder");
			Integer order2 = (Integer) option2.get("methodOrder");

			// Methods where the order field is defined go before those where it
			// is not
			if (order1 != null && order2 == null) {
				return -1;
			}
			if (order2 != null && order1 == null) {
				return 1;
			}

			// If both define an order, the lower order goes first
			if (order1 != null && order2 != null) {
				if (order1.intValue() < order2.intValue()) {
					return -1;
				}
				if (order2.intValue() < order1.intValue()) {
					return 1;
				}
			}

			// If neither define an order or if the orders are equal, go with
			// the price of the option
			Double rate1 = (Double) option1.get("priceDouble");
			Double rate2 = (Double) option2.get("priceDouble");
			if (rate1 == null || rate2 == null) {
				return 0;
			}
			if (rate1.doubleValue() < rate2.doubleValue()) {
				return -1;
			} else if (rate1.doubleValue() == rate2.doubleValue()) {
				return 0;
			} else {
				return 1;
			}
		} catch (Exception e) {
			return 0;
		}
	}
}
