package com.softslate.commerce.businessobjects.shipping;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a discount range within the system.
 * <code>ShippingRuleRangeBean</code> is the default implementation of the
 * <code>ShippingRuleRange</code> interface for the application.
 * <p>
 * The database stores discount ranges in the <code>sscShippingRuleRange</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>ShippingRuleRangeBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author Jason McWilliams
 * 
 */
@Audited
@AuditTable("AudShippingRuleRange")
public class ShippingRuleRangeBean extends BaseBusinessObject implements
		ShippingRuleRange, Serializable {

	private static final long serialVersionUID = 6361036916924349236L;

	private int shippingRuleRangeID = 0;

	private int shippingRuleID = 0;

	private Double floor = new Double(0);

	private Double ceiling = new Double(0);

	private boolean isActive = true;

	private String discountType = null;

	private Double discount = null;

	private String pkgDescription = null;

	private Double pkgWeight = new Double(0);

	private Double pkgLength = new Double(0);

	private Double pkgHeight = new Double(0);

	private Double pkgWidth = new Double(0);

	private String pkgExtra1 = null;

	private String pkgExtra2 = null;

	private String pkgExtra3 = null;

	private String pkgExtra4 = null;

	private String pkgExtra5 = null;

	private String limitMethods = null;

	private String limitType = null;

	private String countries = null;

	private String states = null;

	private ShippingRule shippingRule = null;

	public int getShippingRuleRangeID() {
		return shippingRuleRangeID;
	}

	public void setShippingRuleRangeID(int shippingRuleRangeID) {
		this.shippingRuleRangeID = shippingRuleRangeID;
	}

	public int getShippingRuleID() {
		return shippingRuleID;
	}

	public void setShippingRuleID(int shippingRuleID) {
		this.shippingRuleID = shippingRuleID;
	}

	public Double getFloor() {
		return floor;
	}

	public void setFloor(Double floor) {
		this.floor = floor;
	}

	public Double getCeiling() {
		return ceiling;
	}

	public void setCeiling(Double ceiling) {
		this.ceiling = ceiling;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public String getDiscountType() {
		return discountType;
	}

	public void setDiscountType(String discountType) {
		this.discountType = discountType;
	}

	public Double getDiscount() {
		return discount;
	}

	public void setDiscount(Double discount) {
		this.discount = discount;
	}

	public String getPkgDescription() {
		return pkgDescription;
	}

	public void setPkgDescription(String pkgDescription) {
		this.pkgDescription = pkgDescription;
	}

	public Double getPkgWeight() {
		return pkgWeight;
	}

	public void setPkgWeight(Double pkgWeight) {
		this.pkgWeight = pkgWeight;
	}

	public Double getPkgLength() {
		return pkgLength;
	}

	public void setPkgLength(Double pkgLength) {
		this.pkgLength = pkgLength;
	}

	public Double getPkgHeight() {
		return pkgHeight;
	}

	public void setPkgHeight(Double pkgHeight) {
		this.pkgHeight = pkgHeight;
	}

	public Double getPkgWidth() {
		return pkgWidth;
	}

	public void setPkgWidth(Double pkgWidth) {
		this.pkgWidth = pkgWidth;
	}

	public String getPkgExtra1() {
		return pkgExtra1;
	}

	public void setPkgExtra1(String pkgExtra1) {
		this.pkgExtra1 = pkgExtra1;
	}

	public String getPkgExtra2() {
		return pkgExtra2;
	}

	public void setPkgExtra2(String pkgExtra2) {
		this.pkgExtra2 = pkgExtra2;
	}

	public String getPkgExtra3() {
		return pkgExtra3;
	}

	public void setPkgExtra3(String pkgExtra3) {
		this.pkgExtra3 = pkgExtra3;
	}

	public String getPkgExtra4() {
		return pkgExtra4;
	}

	public void setPkgExtra4(String pkgExtra4) {
		this.pkgExtra4 = pkgExtra4;
	}

	public String getPkgExtra5() {
		return pkgExtra5;
	}

	public void setPkgExtra5(String pkgExtra5) {
		this.pkgExtra5 = pkgExtra5;
	}

	public String getLimitMethods() {
		return limitMethods;
	}

	public void setLimitMethods(String limitMethods) {
		this.limitMethods = limitMethods;
	}

	public String getLimitType() {
		return limitType;
	}

	public void setLimitType(String limitType) {
		this.limitType = limitType;
	}

	public ShippingRule getShippingRule() {
		return shippingRule;
	}

	public void setShippingRule(ShippingRule shippingRule) {
		this.shippingRule = shippingRule;
	}

	public String getCountries() {
		return countries;
	}

	public void setCountries(String countries) {
		this.countries = countries;
	}

	public String getStates() {
		return states;
	}

	public void setStates(String states) {
		this.states = states;
	}

}
