package com.softslate.commerce.customer.core;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.octo.captcha.module.config.CaptchaModuleConfigHelper;
import com.octo.captcha.module.struts.CaptchaServicePlugin;
import com.octo.captcha.service.CaptchaService;
import com.softslate.commerce.businessobjects.core.RecaptchaProcessor;

/**
 * Struts form class for submissions to the contact form and other form-to-email
 * forms.
 * <p>
 * The validate method of this class checks to make sure a "formType" parameter
 * is present. The form type is used to look up which fields are validated, and
 * required. They are looked up in the application settings by finding settings
 * whose codes match 'requiredFields&lt;formType&gt;' and
 * 'validatedFields&lt;formType&gt;'.
 * <p>
 * If the "useImageVerificationWithContactForms" setting is set, it checks the
 * entered text against the Jcaptcha image that was presented.
 * 
 * @author David Tobey
 */
public class Form2EmailForm extends BaseForm {
	private static final long serialVersionUID = 3258131362296115257L;

	static Log log = LogFactory.getLog(Form2EmailForm.class);

	private String formType = null;

	public String getFormType() {
		return formType;
	}

	public void setFormType(String formType) {
		this.formType = formType;
	}

	private String textFromImage = null;

	public String getTextFromImage() {
		return textFromImage;
	}

	public void setTextFromImage(String textFromImage) {
		this.textFromImage = textFromImage;
	}

	private String emailerOptInApproval = null;

	public String getEmailerOptInApproval() {
		return emailerOptInApproval;
	}

	public void setEmailerOptInApproval(String emailerOptInApproval) {
		this.emailerOptInApproval = emailerOptInApproval;
	}

	private String[] emailerMultipleListOptInApproval = null;
	
	public String[] getEmailerMultipleListOptInApproval() {
		return emailerMultipleListOptInApproval;
	}

	public void setEmailerMultipleListOptInApproval(
			String[] emailerMultipleListOptInApproval) {
		this.emailerMultipleListOptInApproval = emailerMultipleListOptInApproval;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		// Require the formType
		if (getFormType() == null || getFormType().length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Form type"));
		}

		// Check for required fields
		String requiredFields = (String) getSettings().getValue(
				getFormType() + "RequiredFields");
		if (requiredFields != null && requiredFields.length() > 0) {
			String[] fields = requiredFields.split(",");
			for (int i = 0; i < fields.length; i++) {
				String field = fields[i].trim();
				if (request.getParameter(field) == null
						|| request.getParameter(field).length() == 0) {
					getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.required", field));
				}
			}
		}

		// Check for invalid fields
		String validatedFields = (String) getSettings().getValue(
				getFormType() + "ValidatedFields");
		if (validatedFields != null && validatedFields.length() > 0) {
			String[] fields = validatedFields.split(",");
			for (int i = 0; i < fields.length; i++) {
				String field = fields[i].trim();
				if (request.getParameter(field) != null
						&& request.getParameter(field).length() > 0) {
					if (!isFieldValid(field, request.getParameter(field))) {
						getErrors().add(ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.invalid", field));
					}
				}
			}
		}

		// Validate CAPTCHA
		try {
			String useLegacyCaptchaMethod = (String) getSettings().getValue("useLegacyCaptchaMethod");
			String useImageVerificationWithContactForms = (String) getSettings().getValue(
					"useImageVerificationWithContactForms");
			if (getErrors().isEmpty() && useImageVerificationWithContactForms != null
					&& useImageVerificationWithContactForms.equals("1") && useLegacyCaptchaMethod != null
					&& useLegacyCaptchaMethod.equals("1")) {

				// Legacy JCaptcha method
				if (getTextFromImage() != null) {
					setTextFromImage(getTextFromImage().toLowerCase());
				}
				CaptchaService service = CaptchaServicePlugin.getInstance().getService();
				String captchaID = CaptchaModuleConfigHelper.getId(request);
				if (captchaID == null || getTextFromImage() == null || getTextFromImage().length() == 0
						|| !service.validateResponseForID(captchaID, getTextFromImage()).booleanValue()) {
					setTextFromImage(null);
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidCaptcha"));
				}
			} else if (getErrors().isEmpty() && useImageVerificationWithContactForms != null
					&& useImageVerificationWithContactForms.equals("1")) {

				// Google Recaptcha method
				RecaptchaProcessor rp = getBusinessObjectFactory().createObject(RecaptchaProcessor.class);
				Map<String, String> params = new HashMap();
				params.put("response", request.getParameter("g-recaptcha-response"));
				Map response = rp.verifyRecaptcha(params);
				if (response == null || response.get("success") == null
						|| response.get("success").equals(Boolean.FALSE)) {
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
				}
			}

		} catch (Exception e) {
			log.error(e.toString(), e);
			setTextFromImage(null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	public boolean isFieldValid(String field, String value) {
		if (field.equalsIgnoreCase("email")) {
			return formUtils().isValidEmail(value);
		}
		if (field.equalsIgnoreCase("phone")) {
			return formUtils().isValidPhone(value);
		}
		return true;
	}

	public String constructEmailText(HttpServletRequest request) {

		// Loop through the prompted fields and construct a text message
		StringBuffer text = new StringBuffer("");
		text.append(getMessage("core", getFormType() + ".emailMessageIntro",
				null));
		text.append("\n\n");

		String promptedFields = (String) getSettings().getValue(
				getFormType() + "PromptedFields");
		if (promptedFields != null && promptedFields.length() > 0) {
			String[] fields = promptedFields.split(",");
			for (int i = 0; i < fields.length; i++) {
				String field = fields[i].trim();
				String[] values = request.getParameterValues(field);
				if (values != null) {
					text.append(field);
					text.append(": ");
					for (int j = 0; j < values.length; j++) {
						if (j > 0) {
							text.append("\n");
						}
						String value = values[j].trim();
						text.append(value);
					}
					text.append("\n\n");
				}
			}
		}
		text.append(getMessage("core", getFormType() + ".emailMessageEnding",
				new String[] { request.getServletPath(),
						request.getRemoteAddr() }));
		return text.toString();
	}

	public String constructEmailHTML(HttpServletRequest request) {

		// Loop through the prompted fields and construct an HTML message
		StringBuffer text = new StringBuffer("");
		text.append("<html><body><p>");
		text.append(getMessage("core", getFormType() + ".emailMessageIntro",
				null));
		text.append("</p><dl>");
		String promptedFields = (String) getSettings().getValue(
				getFormType() + "PromptedFields");
		if (promptedFields != null && promptedFields.length() > 0) {
			String[] fields = promptedFields.split(",");
			for (int i = 0; i < fields.length; i++) {
				String field = fields[i].trim();
				String[] values = request.getParameterValues(field);
				if (values != null) {
					text.append("<dt>");
					text.append(field);
					text.append(":</dt>");
					text.append("<dd>");
					for (int j = 0; j < values.length; j++) {
						if (j > 0) {
							text.append("<br/>\n");
						}
						String value = values[j].trim();
						text.append(value);
					}
					text.append("</dd>");
					text.append("\n");
				}
			}
		}
		text.append("</dl><p>");
		text.append(getMessage("core", getFormType() + ".emailMessageEnding",
				new String[] { request.getRequestURL().toString(),
						request.getRemoteAddr() }));
		text.append("</p></body></html>");
		return text.toString();
	}
}
