package com.softslate.commerce.customer.core;

import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.jstl.core.Config;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.tiles.actions.TilesAction;

import com.softslate.commerce.businessobjects.core.User;

/**
 * Action class to handle a request from the user to switch the language that
 * navigational elements, buttons, and labels are displayed with. Corresponds to
 * the url "/SwitchLanguage.do".
 * 
 * The execute() method of this class reads a URL parameter named "language",
 * and and optional parameter named "country", and sets the user's locale
 * accordingly, as well as other locale session variables used by the JSTL and
 * Struts tag libraries.
 * 
 * @author David Tobey
 */
public class SwitchLanguageAction extends TilesAction {
	static Log log = LogFactory.getLog(SwitchLanguageAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (log.isDebugEnabled())
			log.debug("Starting action");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		if (request.getParameter("language") == null) {
			return mapping.findForward("failure");
		}

		synchronized (request.getSession()) {
			User user = (User) request.getSession().getAttribute("user");
			Locale locale = new Locale(request.getParameter("language"));
			if (request.getParameter("country") != null) {
				locale = new Locale(request.getParameter("language"), request
						.getParameter("country"));
			}
			user.setLocale(locale);
			request.getSession().setAttribute(Globals.LOCALE_KEY, locale);
			Config.set(request.getSession(), Config.FMT_LOCALE, locale);
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.languageSwitched"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
		}
		return mapping.findForward("success");
	}

}
