package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Action to handle customer password reset
 * 
 * @author Mike Groves, jmcwill
 */
public class ResetPasswordAction extends BaseAction {

	static Log log = LogFactory.getLog(ResetPasswordAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		
		if (log.isInfoEnabled())
			log.info("Starting action.");
		ResetPasswordForm baseForm = (ResetPasswordForm) form;
		
		String useAdvancedLostPassword = (String) baseForm.getSettings().getValue("useAdvancedLostPassword");
		if (!useAdvancedLostPassword.equals("1")) {
			response.setStatus(HttpServletResponse.SC_NOT_FOUND);
			return mapping.findForward("failureOff");
		}

		CustomerProcessor customerProcessor = (CustomerProcessor) baseForm.getBusinessObjectFactory().createObject(
				"customerProcessorImplementer");
		Map result = customerProcessor.loadCustomerFromResetToken(PropertyUtils.describe(baseForm));
		
		if (result != null && result.get("resultCode") != null
				&& !result.get("resultCode").equals("0")) {
			String resultCode = (String) result.get("resultCode");
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage(resultCode));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): "
						+ baseForm.getErrors().toString());
			return mapping.findForward("failure");
		}
		
		Customer customer = (Customer) result.get("customer");
		if (customer == null || customer.getPasswordToken() == null) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.passwordNotFound"));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
			return mapping.findForward("failure");
		}
		
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		return mapping.findForward("success");
	}
}
