package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.AddressesForm;
import com.softslate.commerce.customer.order.BillingAddressForm;

/**
 * Struts action class used for requests to update only the billing address
 * information for a customer account. Used for requests to
 * "/UpdateBillingAddress.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#updateAddresses(Map)}
 * to process the request and update the information in the database. It also
 * sets the session attribute "shippingAndTaxesProcessed" to "false", in case
 * the user has gone part way through the checkout process, since a chnage of
 * the address may affect the tax and shipping calculations.
 * 
 * @author Jason McWilliams
 * 
 */
public class UpdateBillingAddressAction extends BaseAction {
	static Log log = LogFactory.getLog(UpdateBillingAddressAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BillingAddressForm baseForm = (BillingAddressForm) form;
		synchronized (request.getSession()) {
			request.getSession().setAttribute("shippingAndTaxesProcessed",
					"false");

			// Preserve the original request so the customer is redirected
			// back to the main addresses page after logging in
			request.setAttribute("originalRequest", "/AccountAddresses.do");
			
			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm.getBusinessObjectFactory().createObject(
					"customerProcessorImplementer");
			AddressesForm addressesForm = new AddressesForm();
			boolean copied = false;
			Map result;
			
			// Fill in delivery info with default billing info if they don't
			// have any because it will create a blank one when adding to db
			CustomerAddress customerAddress = baseForm.getUser().getCustomer().getFirstAddress();
			if (customerAddress == null) {
				BeanUtils.copyProperties(addressesForm, baseForm);
				addressesForm.setDeliveryUseBilling(true);
				addressesForm.copyBillingToDelivery();
				copied = true;
				Map properties = PropertyUtils.describe(addressesForm);
				properties.remove("paymentToken");
				properties.remove("creditCardName");
				properties.remove("creditCardType");
				properties.remove("creditCardDate");
				properties.remove("creditCardNumberDisplay");
				properties.remove("identifier");
				result = customerProcessor.updateAddresses(properties);
			} else {
				Map properties = PropertyUtils.describe(baseForm);
				properties.remove("paymentToken");
				properties.remove("creditCardName");
				properties.remove("creditCardType");
				properties.remove("creditCardDate");
				properties.remove("creditCardNumberDisplay");
				properties.remove("identifier");
				result = customerProcessor.updateAddresses(properties);
			}
			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				String resultCode = (String) result.get("resultCode");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage(resultCode));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}
			
			// Site uses AddressForm in most places...
			if (!copied) {
				BeanUtils.copyProperties(addressesForm, baseForm);
			}
			baseForm.getRequest().setAttribute("addressForm", addressesForm);
			
			// Set up the state and 'Other' fields for presentation.
			baseForm.prepareStateAndOtherFields(baseForm.getUser()
					.getCustomer());
					
			// Add a message to tell the user it worked
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.addressesUpdated"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			return mapping.findForward("success");
		}
	}
}
