package com.softslate.commerce.customer.payment.authorizeNetCIM;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMPaymentProfileMin;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMProcessor;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResult;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResultCreateProfile;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResultGetProfile;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for submissions of payment information for Authorize.net
 * CIM during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMProcessor}
 * payment processor.
 * 
 * @author Jason McWilliams
 */
public class AuthorizeNetCIMForm extends BaseForm {

	private static final long serialVersionUID = 9144509955375520836L;

	static Log log = LogFactory.getLog(AuthorizeNetCIMForm.class);

	private String cimPaymentProfileID = null;

	private String description = null;

	private String invoiceNumber = null;

	private String purchaseOrderNumber = null;

	private String creditCardNumberDisplay = null;

	private String cimAnon = null;

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getInvoiceNumber() {
		return invoiceNumber;
	}

	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}

	public String getPurchaseOrderNumber() {
		return purchaseOrderNumber;
	}

	public void setPurchaseOrderNumber(String purchaseOrderNumber) {
		this.purchaseOrderNumber = purchaseOrderNumber;
	}

	public String getCimPaymentProfileID() {
		return cimPaymentProfileID;
	}

	public void setCimPaymentProfileID(String cimPaymentProfileID) {
		this.cimPaymentProfileID = cimPaymentProfileID;
	}

	private String cimPaymentProfileType = null;

	public String getCimPaymentProfileType() {
		return cimPaymentProfileType;
	}

	public void setCimPaymentProfileType(String cimPaymentProfileType) {
		this.cimPaymentProfileType = cimPaymentProfileType;
	}

	public String getCreditCardNumberDisplay() {
		return creditCardNumberDisplay;
	}

	public void setCreditCardNumberDisplay(String creditCardNumberDisplay) {
		this.creditCardNumberDisplay = creditCardNumberDisplay;
	}

	public String getCimAnon() {
		return cimAnon;
	}

	public void setCimAnon(String cimAnon) {
		this.cimAnon = cimAnon;
	}

	/**
	 * Called after this form is populated. It is called before validating if it
	 * is being validated. We use this to get a token from authorize.net to show
	 * the hosted form.
	 * 
	 * @param validating
	 *            Is the form being validated or just populated and shown to the
	 *            user.
	 */
	public void preparePaymentFormHook(Boolean validating, BaseForm baseForm) throws Exception {

		// Will set the cimToken in the request scope. If it exists, the user is
		// allowed to work with CIM, otherwise they will have to use another
		// method.  Only get cimToken if not using API.

		// No need to do anything if using the API, it will be done using ajax
		String authorizeNetCIMHostedMethod = (String) baseForm.getSettings().getValue("authorizeNetCIMHostedMethod");
		boolean useAPI = false;
		if ("API Only".equals(authorizeNetCIMHostedMethod)){
			useAPI = true;
		}
		if (!validating) {
			if (baseForm != null && baseForm.getRequest() != null) {
				synchronized (baseForm.getRequest().getSession()) {
					String profileType = "customer";
					Customer customer = null;
					Order order = null;
					if (baseForm.getUser() != null && baseForm.getUser().getCustomer() != null) {
						customer = baseForm.getUser().getCustomer();
					}
					if (baseForm.getUser() != null && baseForm.getUser().getOrder() != null) {
						order = baseForm.getUser().getOrder();
					}
					AuthorizeNetCIMProcessor cimProcessor =  baseForm
							.getBusinessObjectFactory().createObject(AuthorizeNetCIMProcessor.class);
					cimProcessor.initialize();
					String internalErrorMessage = null;
					List<Map<String, String>> messages = null;
					String validProfileIDStr = null;
					long deleteID = 0;
					try {
						deleteID = Long.parseLong(baseForm.getRequest().getParameter("deleteID"));
					} catch (Exception ex) {
						deleteID = 0;
					}
					boolean tryAnonymous = false;
					if (customer != null && customer.getCustomerID() > 0) {
						// Valid Customer
						long cimProfileID = 0;
						try {
							cimProfileID = Long.parseLong(customer.getCimProfileID());
						} catch (Exception ex) {
							cimProfileID = 0;
						}
						if (cimProfileID > 0) {
							// Valid profile exists
						} else if (cimProfileID < 0) {
							// Admin has set this as an invalid payment method
							// for this user
							internalErrorMessage = "This payment method is not allowed.";
							logProfileErrors(internalErrorMessage, messages, customer, order, "createCustomerProfile");
						} else {
							// Need to create a profile
							AuthorizeNetCIMResultCreateProfile profile = cimProcessor.createCustomerProfile(customer,
									null, order);
							if (profile.getError()) {
								internalErrorMessage = "Error creating new customer profile.";
								messages = profile.getMessages();
								tryAnonymous = true;
							} else if (profile.getProfileID() > 0) {
								CustomerProcessor customerProcessor =  baseForm
										.getBusinessObjectFactory().createObject(CustomerProcessor.class);
								Map parameters = new HashMap();
								parameters.put("cimProfileID", Long.toString(profile.getProfileID()));
								customerProcessor.updateCustomer(parameters);
								customer.setCimProfileID(Long.toString(profile.getProfileID()));
							} else {
								internalErrorMessage = "No Error, but returning profile id is 0??";
								messages = profile.getMessages();
								tryAnonymous = true;
							}
							logProfileErrors(internalErrorMessage, messages, customer, order, "createCustomerProfile");
						}
						validProfileIDStr = customer.getCimProfileID();
					} else {
						// No customer, try the order record
						if (order != null && StringUtils.isNotBlank(order.getPaymentProfileID())) {
							validProfileIDStr = order.getPaymentProfileID();
						}
						tryAnonymous = true;
						profileType = "order";
					}
					long validProfileID = 0;
					try {
						validProfileID = Long.parseLong(validProfileIDStr);
					} catch (Exception ex) {
						validProfileID = 0;
					}

					// Try anonymous
					if (validProfileID < 1 && tryAnonymous) {
						internalErrorMessage = null;
						messages = null;
						AuthorizeNetCIMResultCreateProfile profile = cimProcessor.createCustomerProfile(order);
						if (profile.getError()) {
							// Throw Exception?
							internalErrorMessage = "Error creating new order payment profile.";
							messages = profile.getMessages();
							logProfileErrors(internalErrorMessage, messages, customer, order, "createCustomerProfile");
						} else if (profile.getProfileID() > 0) {
							order.setPaymentProfileID(Long.toString(profile.getProfileID()));
							validProfileID = profile.getProfileID();
						} else {
							internalErrorMessage = "No Error, but returning order payment profile id is 0??";
							messages = profile.getMessages();
							logProfileErrors(internalErrorMessage, messages, customer, order, "createCustomerProfile");
						}
					}
					if (useAPI) {
						// Stop here, we only needed to create the user profile
						if (validProfileID > 0) {
							baseForm.getRequest().setAttribute("cimPaymentProfileType", profileType);
						} else {
							// Give Up, Something is wrong
							if (log.isInfoEnabled())
								log.info("Unable to find or create a valid customer profile ID.");
						}
					} else if (validProfileID > 0) {
						Map<String, String> results = cimProcessor.getHostedProfilePage(validProfileID, "checkout");
						if (results == null) {
							// No results, no good
							if (log.isInfoEnabled())
								log.info("Error getting hosted page: no results from gethostedProfilePage()");
						} else {
							String xmlResultCode = results.get("xmlResultCode");
							String xmlMessageCode = results.get("xmlMessageCode");
							String xmlMessageText = results.get("xmlMessageText");
							String token = results.get("token");
							if ("Ok".equalsIgnoreCase(xmlResultCode) && StringUtils.isNotBlank(token)) {
								baseForm.getRequest().setAttribute("cimToken", token);
								baseForm.getRequest().setAttribute("cimPaymentProfileType", profileType);

								// Now also pull in the current payment methods
								// available to the user
								AuthorizeNetCIMResultGetProfile getProfileResult = cimProcessor
										.getCustomerProfile(validProfileID);
								if (getProfileResult.getError()) {
									logProfileErrors("Error getting profile inforamtion.",
											getProfileResult.getMessages(), customer, order, "getCustomerProfile");
								} else {
									if (getProfileResult.getProfileID() > 0
											&& getProfileResult.getProfileID() == validProfileID) {
										if (getProfileResult.getPaymentProfiles() != null
												&& getProfileResult.getPaymentProfiles().size() > 0) {
											Collection<AuthorizeNetCIMPaymentProfileMin> paymentProfiles = getProfileResult
													.getPaymentProfiles();
											String authorizeNetCIMAllowDeletePayments = (String) baseForm.getSettings()
													.getValue("authorizeNetCIMAllowDeletePayments");
											if (deleteID > 0 && "1".equals(authorizeNetCIMAllowDeletePayments)) {
												for (AuthorizeNetCIMPaymentProfileMin p : paymentProfiles) {
													if (p.getCustomerPaymentProfileId() == deleteID) {
														AuthorizeNetCIMResult deletePaymentResult = null;
														try {
															deletePaymentResult = cimProcessor
																	.deleteCustomerPaymentProfile(validProfileID,
																			deleteID);
														} catch (Exception ex) {
															logProfileErrors("Exception deleting payment profile: "
																	+ ex.getMessage(), null, customer, order,
																	"deleteCustomerPaymentProfile");
														}
														if (getProfileResult.getError()) {
															logProfileErrors("Error deleting payment profile",
																	deletePaymentResult.getMessages(), customer, order,
																	"deleteCustomerPaymentProfile");
														} else {
															paymentProfiles.remove(p);
														}
														break;
													}
												}
											}
											Collections.reverse((List) paymentProfiles);
											baseForm.getRequest().setAttribute("cimPaymentProfiles", paymentProfiles);
										}
									} else {
										logProfileErrors("Returned profileID does not match the sent profileID",
												getProfileResult.getMessages(), customer, order, "getCustomerProfile");
									}
								}
							} else {
								if (log.isInfoEnabled()) {
									log.info("Error getting hosted page - result code : " + xmlResultCode);
									log.info("Error getting hosted page - message code : " + xmlMessageCode);
									log.info("Error getting hosted page - message text : " + xmlMessageText);
									if (StringUtils.isBlank(token)) {
										log.info("Error getting hosted page - token is blank");
									}
								}
							}
						}
					} else {
						// Give Up, Something is wrong
						if (log.isInfoEnabled())
							log.info("Unable to find or create a valid customer profile ID.");
					}
				}
			} else {
				// We have a big problem, throw an exception
				if (log.isInfoEnabled())
					log.info("Request is null.");
				String message = getMessage("payment", "errorCIMMethodNotAvailable", null);
				throw new Exception(message);
			}
		}
	}

	public void logProfileErrors(String internalErrorMessage, List<Map<String, String>> messages, Customer customer,
			Order order, String command) {
		if (log.isInfoEnabled()) {
			if (customer != null && customer.getCustomerID() > 0) {
				log.info("CIM error " + command + " - customer: " + customer.getCustomerID());
			}
			if (order != null && order.getOrderID() > 0) {
				log.info("CIM error " + command + " - order: " + order.getOrderID());
			}
			log.info(StringUtils.stripToEmpty(internalErrorMessage));
			if (messages != null) {
				for (Map<String, String> message : messages) {
					log.info(StringUtils.stripToEmpty(message.get("code")) + ": "
							+ StringUtils.stripToEmpty(message.get("text")));
				}
			}
		}
	}

	public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		if (StringUtils.isBlank(getCimPaymentProfileID())) {
			String message = getMessage("payment", "payment.errorCIMPaymentProfileIDRequired", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message));
		}

		if (!"order".equalsIgnoreCase(getCimPaymentProfileType())
				&& !"customer".equalsIgnoreCase(getCimPaymentProfileType())) {
			String message = getMessage("payment", "payment.errorCIMPaymentProfileTypeRequired", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message));
		}

		// Double check the user chose a valid paymentID and backfill billing if
		// necessary This security stuff will be checked again later, log
		// errors, but don't stop for now
		AuthorizeNetCIMProcessor cimProcessor = null;
		try {
			Map<String, String> params = new HashMap<String, String>();
			params.put("cimPaymentProfileID", getCimPaymentProfileID());
			params.put("cimPaymentProfileType", getCimPaymentProfileType());
			cimProcessor = (AuthorizeNetCIMProcessor) getBusinessObjectFactory().createObject(
					"authorizeNetCIMProcessorImplementer");

			/*
			 * Don't bother with this right now
			 * 
			 * if (request != null && getUser() != null &&
			 * getUser().getCustomer() != null) { synchronized
			 * (request.getSession()) { if
			 * ("true".equalsIgnoreCase(getCimAnon())) { getUser()
			 * .getCustomer() .setCimProfileID( Long
			 * .toString(AuthorizeNetCIMProcessor.CUSTOMER_ANON)); } } }
			 */
			cimProcessor.initialize();
			Map<String, String> results = cimProcessor.validatePaymentMethod(params);
			if (results == null || !"0".equals(results.get("resultCode"))) {
				String resultCode = results.get("resultCode");
				String resultMessage = results.get("resultMessage");
				String className = cimProcessor.getClass().getName().replaceFirst("(.*)\\.", "");
				String message = getMessage("payment", "payment.error" + className + resultCode, null);
				if (message == null || message.length() == 0) {
					if (resultMessage == null) {
						message = getMessage("payment", "payment.processingError", null);
					} else {
						message = getMessage("payment", "payment.processingErrorArg", new String[] { resultMessage });
					}
				}
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message));
			} else {
				setCreditCardNumberDisplay((String) results.get("creditCardNumberDisplay"));
			}

		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Exception trying to validate CIM payment data: " + ex);
			}
		}

		ActionUtils actionUtils = (ActionUtils) getBusinessObjectFactory().createUtilsObject("actionUtilsImplementer");
		// Populate description, comments, etc., parsing them for variables.
		try {
			setDescription(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetCIMDescription")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setInvoiceNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetCIMInvoiceNum")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setPurchaseOrderNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("authorizeNetCIMPONumber")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		if (log.isDebugEnabled()) {
			log.debug("description: " + getDescription());
			log.debug("invoice: " + getInvoiceNumber());
			log.debug("ponum: " + getPurchaseOrderNumber());
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}

}
