package com.softslate.commerce.customer.product;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * Struts form class that correspondes to the product page. Used for requests to
 * "/Product.do".
 * <p>
 * The <code>validate</code> method of this class simply makes sure that a
 * product <code>code</code> of non-zero length was submitted.
 * <p>
 * Alternatively, if a request attribute named "errorProductCode" is set, this
 * class will populate itself with the value of that variable. This is used in
 * cases where an error occurs during cart processing and the failure action is
 * to forward the error to the appropriate product screen.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.product.ProductAction
 */
public class ProductForm extends BaseDynaForm {

	private static final long serialVersionUID = 3904960858760033585L;

	static Log log = LogFactory.getLog(ProductForm.class);

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		if (getRequest().getAttribute("errorProductCode") != null) {
			set("code", getRequest().getAttribute("errorProductCode"));
		}
		if (!getFormUtils().stringExists(this, "code")) {
			String message = getMessage("product", "product.code", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

}
