package com.softslate.commerce.customer.product;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.ProductList;
import com.softslate.commerce.businessobjects.product.ProductListBean;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * Struts action class that corresponds to the search results page. Used for
 * requests to "/Search.do".
 * <p>
 * The <code>execute</code> method of this class retrieves information from the
 * database for the list of products in the system matching a given search
 * string and puts it in the request scope for use by the view layer. Given a
 * {@link com.softslate.commerce.customer.product.SearchForm} with its
 * <code>searchString</code> property populated, it creates and initializes an
 * instance of
 * {@link com.softslate.commerce.businessobjects.product.ProductList}. It then
 * invokes
 * {@link com.softslate.commerce.businessobjects.product.ProductProcessor#getProductList(ProductList)}
 * to load the product list information from the database. It puts the
 * initialized <code>ProductList</code> object in the request scope as an
 * attribute with the name "productList".
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.product.SearchForm
 * @see com.softslate.commerce.businessobjects.product.ProductList
 * @see com.softslate.commerce.businessobjects.product.ProductProcessor
 */
public class SearchAction extends BaseAction {

	static Log log = LogFactory.getLog(SearchAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseDynaForm baseForm = (BaseDynaForm) form;

		// Create a processor to use to load the products
		ProductProcessor productProcessor = (ProductProcessor) baseForm.getBusinessObjectFactory().createObject(
				"productProcessorImplementer");
		ProductList productList = (ProductList) baseForm.getBusinessObjectFactory().createObject(
				"productListImplementer");
		BeanUtils.copyProperties(productList, baseForm);
		String itemsPerPage = (String) baseForm.getSettings().getValue("searchResultsPageSize");

		// 'View all' support
		if (baseForm.getMap().get("firstRow") != null && baseForm.getMap().get("firstRow").equals("all")) {
			itemsPerPage = (String) baseForm.getSettings().getValue("maxRecordsProcessed");
			baseForm.getMap().put("firstRow", "1");
		}
		if (log.isDebugEnabled())
			log.debug("itemsPerPage: " + itemsPerPage);
		productList.setItemsPerPage(Integer.parseInt(itemsPerPage));
		productList.setProductListType(ProductListBean.SEARCH);

		// Load the products and place the list in the request scope
		productList = productProcessor.getProductList(productList);
		request.setAttribute("productList", productList);
		request.setAttribute("itemCount", new Integer(productList.getItemCount()));
		request.setAttribute("itemsPerPage", new Integer(productList.getItemsPerPage()));
		request.setAttribute("firstRow", new Integer(productList.getFirstRow()));
		request.setAttribute("listCollection", productList.getProductListCollection());
		String searchString = request.getParameter("searchString");
		request.setAttribute("linkBase", "/Search.do?searchString=" + java.net.URLEncoder.encode(searchString, "UTF-8")
				+ "&amp;");

		return mapping.findForward("success");
	}

}
