package com.softslate.commerce.daos.email;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.email.Email;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;


/**
 * Generic DAO to manage the <code>sscEmail</code> table of the
 * database.
 */
public class EmailDAOHibernate extends BaseDAO implements DAOInterface, EmailDAO {

    static Log log = LogFactory.getLog(EmailDAOHibernate.class);

    private Email email = null;

    public Email getEmail() {
        return (this.email);
    }

    public void setEmail(Email email) {
        this.email = email;
    }

    public void loadEmailFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            Email record = (Email) session.get(
                    Email.class, new Integer(getEmail()
                            .getEmailID()));
            PropertyUtils.copyProperties(getEmail(), record);
            setEmail(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertEmail() throws Exception {
        insertEmail(false);
    }

    public void insertEmail(boolean commit) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
			getEmail().setCreatedTime(formatDateTime(new java.util.Date()));
			getEmail().setLastActionTime(formatDateTime(new java.util.Date()));
            session.save(getEmail());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
			if (getEmail() != null) {
				session.evict(getEmail());
			}
			loadEmailFromID();
        } catch (HibernateException he) {
            if (log.isErrorEnabled()) log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateEmail() throws DataAccessException {
        updateEmail(false);
    }

    public void updateEmail(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getEmail());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled()) log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteEmail() throws Exception {
        deleteEmail(false);
    }

    public void deleteEmail(boolean commit) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            setEmail((Email) session.load(Email.class,
                    new Integer(getEmail().getEmailID())));
            session.delete(getEmail());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) { log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
