package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.ProductList;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Interface for a DAO that runs miscellaneous queries related to products and
 * categories.
 * 
 * @author David Tobey
 */
public interface ProductGatewayDAO extends DAOInterface {
	abstract Collection loadCategoryTree() throws Exception;

	abstract ProductList loadProductList(ProductList productList) throws Exception;

	abstract Map loadProductsAndCount(Map parameters) throws Exception;

	abstract Map editDeleteProducts(Map parameters, BusinessObjectFactory bof) throws Exception;

	abstract Collection loadAllProducts() throws Exception;

	abstract int loadMaxProductID() throws Exception;

	abstract Collection loadProductsForIndexing(int firstProductID, int lastProductID) throws Exception;

	abstract Collection loadAllProductsSortedByName() throws Exception;

	abstract Collection loadAllProductsSortedByCode() throws Exception;

	abstract Long loadProductCount() throws DataAccessException;

	abstract Collection loadAllProductNamesCodesIDs() throws Exception;

	abstract Collection loadActiveProductNamesCodesIDs() throws Exception;

	abstract Collection processSettings(Map parameters) throws Exception;

	abstract Collection processSettings(boolean commit, Map parameters) throws Exception;

	abstract Collection processCategorySettings(Map parameters) throws Exception;

	abstract Collection processCategorySettings(boolean commit, Map parameters) throws Exception;

	abstract Collection loadProductSettings(Collection productIDs) throws Exception;

	abstract Collection loadSkus(Collection productIDs, Collection attributeIDs) throws Exception;

	abstract void updateProductsActive(Collection productIDs, boolean isActive) throws Exception;

	abstract void updateProductsActive(boolean commit, Collection productIDs, boolean isActive) throws Exception;

	abstract Collection loadAttributeSKUs() throws Exception;

	abstract void updateProductCategoryOrders() throws DataAccessException;

	abstract void updateProductAttributeOrders() throws DataAccessException;

	abstract void updateRelatedProductOrders() throws DataAccessException;

	abstract Collection loadAllProductsInRange(int firstProductID, int lastProductID) throws Exception;

	abstract Collection loadProductsForCategories(Collection categoryCodes, int firstProductID, int lastProductID)
			throws Exception;

	abstract int loadMaxProductSettingID() throws Exception;

	abstract Collection loadAllProductSettingsInRange(int firstID, int lastID) throws Exception;

	abstract int removeCategoryAssociations(String productCode) throws Exception;

	abstract int addCategoryAssociation(String productCode, String categoryCode) throws Exception;

}
